<?php

// These are form buttons that should be labelled in one place or potential problems.
define('MAILOUT_SUBMIT_SAVE_NAME', 'Apply this display name');
define('MAILOUT_SUBMIT_SAVE_HTML', 'Apply this format');
define('MAILOUT_SUBMIT_SAVE_GLOBAL', 'Set global opt-out');
define('MAILOUT_SUBMIT_SAVE_LISTS', 'Save subscriptions');
define('MAILOUT_SUBMIT_SAVE_NEW', 'Add subscriptions');
define('MAILOUT_SUBMIT_SAVE_EMAIL', 'Update email');

define('MAILOUT_SUBMIT_DELETE_FILES', 'Delete selected');
define('MAILOUT_SUBMIT_UPLOAD_FILE', 'Upload file');

define('MAILOUT_UPLOAD_ONLY_ADD', 0);
define('MAILOUT_UPLOAD_ONLY_UPDATE', 1);
define('MAILOUT_UPLOAD_ADD_UPDATE', 2);
define('MAILOUT_UPLOAD_PURGE_LIST', 3);

/**
 * Non-cached admin menu items, called from mailout_menu()
 */
function mailout_admin_menu(&$items) {

  // Lists pages
  if (arg(3) == 'lists') {
    $items[] = array(
      'title' => 'Mailout lists',
      'path' => 'admin/content/mailout/lists/overview',
      'type' => MENU_LOCAL_TASK,
      'callback' => 'mailout_lists',
    );  
    $items[] = array(
      'title' => 'All lists',
      'path' => 'admin/content/mailout/lists/overview/all',
      'type' => MENU_DEFAULT_LOCAL_TASK,
      'callback' => 'mailout_list_subscribers',
      'weight' => -1,
    );
    $all_lists = db_query('SELECT id AS list_id, list_name FROM mailout_list WHERE id > 1 ORDER BY list_name');
    while ($list = db_fetch_array($all_lists)) {
      $items[] = array(
        'title' => $list['list_name'],
        'path' => 'admin/content/mailout/lists/overview/'. $list['list_id'],
        'type' => MENU_LOCAL_TASK,
        'callback' => 'mailout_list_subscribers',
      );
    }
    $items[] = array(
      'title' => 'Global opt-out',
      'path' => 'admin/content/mailout/lists/opt-out',
      'type' => MENU_LOCAL_TASK,
      'callback' => 'mailout_list_subscribers',
      'callback arguments' => array(1),
      'weight' => 1,
    );
    $items[] = array(
      'title' => 'New list',
      'path' => 'admin/content/mailout/lists/add',
      'type' => MENU_LOCAL_TASK,
      'callback' => 'drupal_get_form',
      'callback arguments' => array('mailout_list_edit_form'),
    );
    if (arg(5)) {
      $items[] = array(
        'title' => 'Edit Mailout list',
        'path' => 'admin/content/mailout/lists/edit/'. arg(5),
        'type' => MENU_CALLBACK,
        'callback' => 'drupal_get_form',
        'callback arguments' => array('mailout_list_edit_form'),
      );
      $items[] = array(
        'title' => 'Edit Mailout list',
        'path' => 'admin/content/mailout/lists/overview/'. arg(5),
        'type' => MENU_CALLBACK,
        'callback' => 'mailout_list_subscribers',
      );
      $items[] = array(
        'title' => 'Delete Mailout list',
        'path' => 'admin/content/mailout/lists/delete/'. arg(5),
        'callback' => 'drupal_get_form',
        'type' => MENU_CALLBACK,
        'callback arguments' => array('mailout_list_delete_form'),
      );
      if (arg(6)) {
        $items[] = array(
          'title' => 'Edit Mailout list subscriber',
          'path' => 'admin/content/mailout/lists/edit/sub/'. arg(6),
          'type' => MENU_CALLBACK,
          'callback' => 'drupal_get_form',
          'callback arguments' => array('mailout_list_sub_edit_form'),
        );
      }
    }
  }
  // Stats pages
  if (arg(3) == 'stats') {
    $items[] = array(
      'path' => 'admin/content/mailout/stats/summary/all',
      'title' => t('Mailout stats'),     
      'callback' => 'mailout_stats',
      'type' => MENU_DEFAULT_LOCAL_TASK,
      'weight' => -2,
    );   
    if (arg(5)) {
      $items[] = array(
        'title' => t('List'),     
        'path' => 'admin/content/mailout/stats/summary/'. arg(5),
        'type' => MENU_LOCAL_TASK,
        'callback' => 'mailout_stats_summary',
      );
      $items[] = array(
        'title' => t('Delete'),     
        'path' => 'admin/content/mailout/stats/delete/'. arg(5),
        'callback' => 'drupal_get_form',
        'callback arguments' => array('mailout_stats_delete_form', arg(5)),
      );
      $items[] = array(
        'title' => t('Process queue'),     
        'path' => 'admin/content/mailout/stats/cron/'. arg(5),
        'type' => MENU_LOCAL_TASK,
        'callback' => 'mailout_stats_cron',
        'weight' => 5,
      );
    }
  }
  // Templates
  if (arg(3) == 'templates') {
    $items[] = array(
      'title' => t('New template'),
      'description' => t('Create a new Mailout email template.'),
      'path' => 'admin/content/mailout/templates/new',
      'callback' => 'drupal_get_form',
      'callback arguments' => array('mailout_template_edit_form'),
    );
    $items[] = array(
      'path' => 'admin/content/mailout/templates/edit/'. arg(5),
      'title' => t('Edit Mailout template'),     
      'type' => MENU_CALLBACK,
      'callback' => 'drupal_get_form',
      'callback arguments' => array('mailout_template_edit_form'),
    );   
    $items[] = array(
      'path' => 'admin/content/mailout/templates/preview/'. arg(5),
      'title' => t('Preview Mailout template'),     
      'type' => MENU_CALLBACK,
      'callback' => 'mailout_template_preview',
    );   
    $items[] = array(
      'path' => 'admin/content/mailout/templates/delete/'. arg(5),
      'title' => t('Delete mailout template'),     
      'callback' => 'drupal_get_form',
      'callback arguments' => array('mailout_template_delete_form'),
      'type' => MENU_CALLBACK,
    );
  }
}

// ---------------------------------------------------------------------------
// Mailout list admin

function mailout_overview() {  
  $menu = menu_get_item(NULL, 'admin/content/mailout');
  $content = system_admin_menu_block($menu);
  $output = theme('admin_block_content', $content);
  return $output;
}

/**
 * A summary of a  mailing list.
 */
function mailout_list_subscribers() {
  if (arg(4) == 'opt-out' || !is_numeric(arg(5))) {
    $list_id = MAILOUT_GLOBAL_OPTOUT;
  }
  else {
    $list_id = arg(5);
  }

  // Add a form for adding subscribers to the current list
  if ($list_id != MAILOUT_GLOBAL_OPTOUT) {
    $add_subscribers_form = drupal_get_form('mailout_list_subscribers_add_form');
    $add_subscribers_form .= drupal_get_form('mailout_list_subscribers_file_add_form');
    $add_subscribers_form = theme('fieldset', array('#title' => t('Add subscribers'), '#collapsible' => TRUE, '#collapsed' => TRUE, '#value' => $add_subscribers_form));
  }

  // Add a form for searching subscribers in the current list.
  $search_text = (arg(6)) ? arg(6) : NULL;
  // Not sure if this is right way, but $_GET['q'] won't work because it will have extra values sometimes.
  $return = arg(0) .'/'. arg(1) .'/'. arg(2) .'/'. arg(3) .'/'. arg(4) .'/'. arg(5);
  $search_subscribers_form = drupal_get_form('mailout_subscribers_search_form', $return, $search_text);

  $name = mailout_list_get_list_name($list_id);

  if ($list_id == MAILOUT_GLOBAL_OPTOUT) {
    $header = array(
      array('data' => t('Subscriber'), 'field' => 'm.name', 'sort' => 'asc'),
      array('data' => t('Mail'), 'field' => 'm.email'),
      array('data' => t('User'), 'field' => 'u.name'),
    );
  }
  else {
    $header = array(
      array('data' => t('Subscriber'), 'field' => 'm.name', 'sort' => 'asc'),
      array('data' => t('Mail'), 'field' => 'm.email'),
      array('data' => t('User'), 'field' => 'u.name'),
      t('Status'), t('HTML'),
    );
  }

  // Print a list of users and email addresses
  $sql = 'SELECT DISTINCT m.*, u.uid, u.name AS uname FROM {mailout_list_addresses} m LEFT JOIN {users} u ON m.email = u.mail WHERE list_id = %d';
  $params[] = $list_id;
  if ($search_text) {
    $sql .= ' AND (m.name LIKE \'%%%s%%\' OR m.email LIKE \'%%%s%%\' OR u.name LIKE \'%%%s%%\') ';
    $params = array($list_id, $search_text, $search_text, $search_text);
  }
  $sql .= tablesort_sql($header);

  $result = pager_query($sql, 50, 0, NULL, $params);
  while ($subscriber = db_fetch_array($result)) {
    if ($list_id == MAILOUT_GLOBAL_OPTOUT) {
 //     $ops = l('Edit', 'admin/content/mailout/lists/edit/sub/'. $subscriber['id']);
      $rows[] = array(l($subscriber['name'], 'admin/content/mailout/lists/edit/sub/'. $subscriber['id']), $subscriber['email'], l($subscriber['uname'], 'user/'. $subscriber['uid']));
    }
    else {
      if (mailout_in_list($subscriber['email'])) {
        $optout = theme('image', 'misc/watchdog-error.png', t('Opted out'), t('User has opted out of all emails'));
      }
      else {
        $optout = theme('image', 'misc/watchdog-ok.png', t('OK'), t('User is subscribed and has not opted-out.'));
      }
//      $ops = l('Edit', 'admin/content/mailout/lists/edit/sub/'. $subscriber['id']);
      $rows[] = array(l($subscriber['name'], 'admin/content/mailout/lists/edit/sub/'. $subscriber['id']), $subscriber['email'], l($subscriber['uname'], 'user/'. $subscriber['uid']), $optout, (($subscriber['html']) ? 'Yes' : 'No'));
    } 
  }
  return $add_subscribers_form . $search_subscribers_form . theme('table', $header, $rows) . theme('pager', NULL, 50, 0, array());
}

function mailout_list_subscribers_add_form() {
  $form['add_form'] = array(
    '#type' => 'fieldset',
    '#title' => t('Paste addresses'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    'add_emails_text' => array(
      '#type' => 'textarea',
      '#title' => 'E-mail addresses',
      '#description' => t('Enter some e-mail addresses, one per line, or separated by commas or semi-colons. You can enter a name too, provided you wrap the email address in brackets, for example "John Citizen" &lt;john@citizen.com&gt;.'),
      '#rows' => 4,
    ),
    'add_submit' => array(
        '#type' => 'submit',
        '#value' => t('Add addresses'),
    ),
  );

  return $form;
}


function mailout_list_subscribers_add_form_submit($form_id, $form_values) {
  $addresses = mailout_get_addresses_from_text($form_values['add_emails_text']);
  foreach ($addresses AS $email => $data) {
    if (mailout_in_list($email_address, $list_id)) {
      drupal_set_message(mailout_list_update_subscriber($email, $data['name'], $data['html'], arg(5)));
    }
    else {
      // New subscriber
      drupal_set_message(mailout_list_add_subscriber($email, $data['name'], $data['html'], arg(5)));
    }
  }
}

function mailout_list_subscribers_file_add_form() {
  // This form is a file upload form.
  $form['#attributes']['enctype'] = 'multipart/form-data';

  $form['file_add'] = array(
    '#type' => 'fieldset',
    '#title' => t('Upload file'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  // Form for uploading new files.
  $form['file_add']['upload'] = array(
    '#type' => 'file',
    '#title' => t('Upload file'),
    '#size' => 40,
    '#description' => t('Browse and select a file to be used as a mailout attachment.'),
  );
  $form['file_add']['delimiter'] = array(
    '#type' => 'select',
    '#title' => t('File type'),
    '#multiple' => FALSE,
    '#options' => array('csv' => 'CSV', 'tab' => 'Tab delimited'),
  );
  $form['file_add']['enclosure'] = array(
    '#type' => 'select',
    '#title' => t('String enclosure'),
    '#multiple' => FALSE,
    '#options' => array('double' => '"Double quotes"', 'single' => '\'Single quotes\''),
  );
  $form['file_add']['action'] = array(
    '#type' => 'select',
    '#title' => t('Action'),
    '#multiple' => FALSE,
    '#options' => array(MAILOUT_UPLOAD_ONLY_ADD => t('Only add'), MAILOUT_UPLOAD_ONLY_UPDATE => t('Only update'), MAILOUT_UPLOAD_ADD_UPDATE => t('Add and update'), MAILOUT_UPLOAD_PURGE_LIST => t('Delete ALL, then add')),
  );
  $form['file_add']['col_email'] = array(
    '#type' => 'textfield',
    '#title' => t('Email column'), 
    '#size' => 4,
    '#maxlength' => 2,
    '#default_value' => 1,
    '#description' => t('Specify which column contains the subscriber email address.'),
  );
  $form['file_add']['col_name'] = array(
    '#type' => 'textfield',
    '#title' => t('Name column'),
    '#size' => 4,
    '#maxlength' => 2,
    '#default_value' => 2,
    '#description' => t('Specify which column contains the subscriber name. 0 for none.'),
  );
  $form['file_add']['col_format'] = array(
    '#type' => 'textfield',
    '#title' => t('Format column'), 
    '#size' => 4,
    '#maxlength' => 2,
    '#default_value' => 0,
    '#description' => t('Specify which column contains \'HTML\' or \'Plain\' email format preference. 0 for none.'),
  );
  $form['file_add']['header'] = array(
    '#type' => 'checkbox',
    '#title' => t('Has header row'),
  );
  $form['file_add']['test'] = array(
    '#type' => 'checkbox',
    '#title' => t('TEST: Display a list of found addresses, don\'t save anything.'),
  );
  $form['file_add']['submit'] = array(
    '#type' => 'submit',
    '#value' => t(MAILOUT_SUBMIT_UPLOAD_FILE),
  );
  
  return $form;
}


function mailout_list_subscribers_file_add_form_submit($form_id, $form_values) {
  $file = file_check_upload('upload');
  $addresses = array();

  $delimiter = ($form_values['delimiter'] == 'tab') ? "\t" : ',';
  $enclosure = ($form_values['enclosure'] == 'single') ? '\'' : '"';
  $col_email = $form_values['col_email'] - 1;
  $col_name = $form_values['col_name'] - 1;
  $col_format = $form_values['col_format'] - 1;
  $action = $form_values['action'];
  $list_id = arg(5);
  $format_save = array('plain' => MAILOUT_FORMAT_PLAIN, 'html' => MAILOUT_FORMAT_HTML);

  $address_list = array();
  // Check for a new file upload.
  if ($file = file_check_upload('upload')) {
    // Open file and add all addresses to $address_list array
    $handle = fopen($file->filepath, "r");
    $rows = 0;
    while (($data = fgetcsv($handle, NULL, $delimiter, $enclosure)) !== FALSE) {
      if ($rows == 0 && $form_values['header']) {
        // Nothing with header.
      }
      else {
        $format = strtolower($data[$col_format]);
        $address_list[$data[$col_email]] = array(
          'name' => $data[$col_name],
          'email' => $data[$col_email],
          'html' => (in_array($format, array('html', 'plain'))) ? $format_save[$format] : $format_save['html'],
        );
      }
      $rows++;
    }
  }

  // Print test information if requested.
  if ($form_values['test']) {
    $format_save = array_flip($format_save);
    foreach ($address_list AS $subdata) {
      drupal_set_message($subdata['name'] .' - '. $subdata['email'] .' - '. $format_save[$subdata['html']]);
    }
    return;
  }

  // Print test information if requested.
  if ($action == MAILOUT_UPLOAD_PURGE_LIST) {
    db_query('DELETE FROM {mailout_list_addresses} WHERE list_id = %d', $list_id);
    drupal_set_message(t('Removed all existing subscriptions from the list'));
    $action = MAILOUT_UPLOAD_ONLY_ADD;
  }
  
  foreach ($address_list AS $subdata) {
    // Verify email
    if ($message = user_validate_mail($subdata['email'])) {
      drupal_set_message(t('Skipped "@email" as it appears invalid.', array('@email' => $subdata['email'])), 'warning');
      continue;
    }
    if (mailout_in_list($subdata['email'], $list_id)) {
      // Email is already in list - update.
      if ($action == MAILOUT_UPLOAD_ONLY_UPDATE || $action == MAILOUT_UPLOAD_ADD_UPDATE) {
        mailout_list_update_subscriber($subdata['email'], $subdata['name'], $subdata['html'], $list_id);
        drupal_set_message(t('Updated: @email.', array('@email' => $subdata['email'])));
      }
    }
    else {
      // Email is not in list - add.
      if ($action == MAILOUT_UPLOAD_ONLY_ADD || $action == MAILOUT_UPLOAD_ADD_UPDATE) {
        mailout_list_add_subscriber($subdata['email'], $subdata['name'], $subdata['html'], $list_id);
        drupal_set_message(t('Added: @email.', array('@email' => $subdata['email'])));
      }
    }
  }
}

/**
 * A search form to limit a list to users that match a search string.
 * I've made this a little dynamic to be re-used on the stats pages and the lists pages.
 */
function mailout_subscribers_search_form($return, $search_text = '') {
  $form['search_form'] = array(
    '#type' => 'fieldset',
    '#title' => t('Search'),
    '#collapsible' => TRUE,
    '#collapsed' => ($search_text) ? FALSE : TRUE,
    'search_return' => array(
      '#type' => 'hidden',
      '#value' => $return,
    ),
    'search_text' => array(
      '#type' => 'textfield',
      '#title' => t('Search e-mail or name'),
      '#required' => TRUE,
      '#default_value' => $search_text,
    ),
    'search_submit' => array(
      '#type' => 'submit',
      '#value' => 'Search',
    ),
  );
  return $form;
}

function mailout_subscribers_search_form_submit($form_id, $form_values) {
  return $form_values['search_return'] .'/'. drupal_urlencode($form_values['search_text']);
}

/**
 * An overview of all mailing lists that are set up.
 */
function mailout_lists() {
  foreach (mailout_get_lists() AS $list) {
    $subscribers = db_result(db_query("SELECT COUNT(id) FROM mailout_list_addresses WHERE list_id = %d GROUP BY list_id", $list->id ));
    $rows[] = array(
      l($list->list_name, 'admin/content/mailout/lists/overview/'. $list->id),
      $subscribers,
      ($list->public) ? t('Yes') : t('No'),
      l(t('Edit'), 'admin/content/mailout/lists/edit/'. $list->id) .', '. l(t('Delete'), 'admin/content/mailout/lists/delete/'. $list->id) .', '. l(t('Subscribers'), 'admin/content/mailout/lists/overview/'. $list->id)
    );
  } 
  return theme('table', array(t('List name'), t('Subscribers'), t('Public'), t('Operations')), $rows);
}

/**
 * Mailout list edit form
 */
function mailout_list_edit_form() {
  $list_id = arg(5);  

  $all_lists = mailout_get_lists();  
  if (!isset($all_lists[$list_id])) {
    drupal_set_title('New list');
    $list_id = 'new';
  }
  else {
    drupal_set_title('Update list');
  }
  $form['list_id'] = array('#type' => 'value', '#value' => $list_id);
  $form['list_name'] = array('#type' => 'textfield', '#title' => t('Mailing list name'), '#default_value' => $all_lists[$list_id]->list_name);
  $form['public'] = array('#type' => 'checkbox', '#title' => t('Public'), '#description' => t('Users will be able to manage their own subscriptions to public lists.'), '#default_value' => $all_lists[$list_id]->public);  
  if ($list_id == 'new') {
    $form['submit'] = array('#type' => 'submit', '#value' => 'Add list');
  }
  else {    
    $form['submit'] = array('#type' => 'submit', '#value' => 'Update');
  }
  return $form;
}

/**
 * Form submission for mailout_list_edit_form
 */
function mailout_list_edit_form_submit($form_id, $form_values) {
  drupal_set_message(mailout_list_update($form_values));
  return 'admin/content/mailout/lists';
}

/**
 * Mailout list edit form, for editing the name, maybe more later.
 */
function mailout_list_delete_form() {
  $list_id = arg(5);  
  $all_lists = mailout_get_lists();  
  $form['list_id'] = array('#type' => 'value', '#value' => $list_id);
  $form['list_name'] = array('#type' => 'value', '#value' => $all_lists[$list_id]->list_name);

  return confirm_form($form, t('Delete Mailout list?'), 'admin/content/mailout/lists', t("The list '%list' will be removed forever.", array('%list' => $all_lists[$list_id]->list_name)), t('Delete'), t('Cancel'));
}

function mailout_list_delete_form_submit($form_id, $form_values) {
  db_query("DELETE FROM {mailout_list} WHERE id = %d", $form_values['list_id']);
  return 'admin/content/mailout/lists'; 
}

/**
 * Mailout list subscriber edit form
 */
function mailout_list_sub_edit_form() {
  $this_sub_id = arg(6);

  // Get this subscription
  $subs = mailout_list_load_subscribers($this_sub_id);
  if (!count($subs)) {
    drupal_set_message('Sorry, the subscription id you entered is no longer valid, please browse the mailing lists.');
    return;
  }
  if (mailout_in_list($subs[$this_sub_id]->email, MAILOUT_GLOBAL_OPTOUT)) {
    $out = TRUE;
  }
  else {
    $out = FALSE;
  }
  
  $global_email = $subs[$this_sub_id]->email;
  
  drupal_set_title('Edit subscriptions for '. $subs[$this_sub_id]->email);
  // Get all subs for this email address
  $subs = mailout_list_load_subscribers(FALSE, $subs[$this_sub_id]->email);

  $problems = mailout_stats_not_sent($subs[$this_sub_id]->email);
  if (count($problems)) {
    $form['apply_to_queues'] = array(
      '#title' => t('Fix queue issues'),
      '#description' => t('If you check this box, and set a new email address below, Mailout will also update corresponding queued emails. This change will only be applied to queued emails that have not been sent'),
      '#type' => 'checkbox',
    );
  }
  
  if (!count($subs)) {
    unset($form['global']['global_name']);
  }

  $global_bestname = ''; $global_bestformat = MAILOUT_FORMAT_HTML;
  foreach ($subs AS $sub_id => $data) {
    $name = mailout_list_get_list_name($subs[$sub_id]->list_id);
    if ($name) {
      $sub_in[] = $subs[$sub_id]->list_id;
      $form['lists']['#tree'] = TRUE;
      $form['lists'][$sub_id] = array(
        'sub_id' => array('#type' => 'hidden', '#value' => $sub_id,),
        'list_id' => array('#type' => 'hidden', '#value' => $subs[$sub_id]->list_id),
        'list_name' => array('#type' => 'hidden', '#value' => $name),
        'list_link' => array('#value' => l($name, 'admin/content/mailout/lists/overview/'. $subs[$sub_id]->list_id),),
        'old_name' => array('#type' => 'hidden', '#value' => $subs[$sub_id]->name,),
        'old_email' => array('#type' => 'hidden', '#value' => $subs[$sub_id]->email,),
        'old_html' => array('#type' => 'hidden', '#value' => $subs[$sub_id]->html),
        'sub_name' => array('#type' => 'textfield', '#size' => 30, '#title' => '', '#default_value' => $subs[$sub_id]->name,),
        'sub_email' => array('#type' => 'textfield', '#size' => 30, '#title' => '', '#default_value' => $subs[$sub_id]->email,),
        'sub_html' => array('#type' => 'select', '#title' => '', '#multiple' => FALSE, '#options' => array(MAILOUT_FORMAT_PLAIN => 'Plain', MAILOUT_FORMAT_HTML => 'HTML'), '#default_value' => $subs[$sub_id]->html),
        'sub_delete' => array('#type' => 'checkbox', '#title' => '',),
      );
      // Simple checks for best name and html format preference.
      $global_bestname = (strlen(mailout_mailname($subs[$sub_id]->name)) > strlen($global_bestname)) ? mailout_mailname($subs[$sub_id]->name) : $global_bestname;
      // Any false will register false (any *0 will end in 0)
      $global_bestformat = $global_bestformat * $subs[$sub_id]->html;
    }
  }

  $form['edited'] = array('#type' => 'hidden', '#value' => $this_sub_id);  
  $form['submit'] = array('#type' => 'submit', '#value' => t(MAILOUT_SUBMIT_SAVE_LISTS));

  // Create an add subscription box.  
  $sub_in[] = MAILOUT_GLOBAL_OPTOUT;
  $not_subscribed = db_query("SELECT * FROM {mailout_list} ml WHERE ml.id NOT IN (". implode(', ', $sub_in) .')');
  $select_other_subs = array();
  while ($list = db_fetch_object($not_subscribed)) {
    $select_other_subs[$list->id] = $list->list_name;
  }
  $form['global_email'] = array('#type' => 'hidden', '#value' => $global_email);
  $form['global_bestname'] = array('#type' => 'hidden', '#value' => $global_bestname);
  $form['global_bestformat'] = array('#type' => 'hidden', '#value' => $global_bestformat);

  $form['display'] = array('#title' => 'Display name', '#tree' => FALSE, '#type' => 'fieldset', '#collapsible' => TRUE, '#collapsed' => TRUE);
  $form['display']['global_name'] = array(
    '#type' => 'textfield', 
    '#title' => t('Global display name'), 
    '#size' => 40, 
    '#description' => ((count($select_other_subs)) ? t('If you\'ve select any new subscriptions below, this name will be used. ') : '') . t('If you enter a new name and click \''. MAILOUT_SUBMIT_SAVE_NAME .'\', you will change the display name for every subscription.'),
  );
  $form['display']['global_name_submit'] = array('#type' => 'submit', '#value' => t(MAILOUT_SUBMIT_SAVE_NAME));

  $form['global'] = array('#title' => 'Global opt-out', '#tree' => FALSE, '#type' => 'fieldset', '#collapsible' => TRUE, '#collapsed' => TRUE);
  $form['global']['global_optout'] = array('#type' => 'checkbox', '#title' => t('Global opt-out'), '#default_value' => $out, '#description' => t('Check this to stop sending emails to this subscriber, without having to deleting subscription info.'));
  if ($out) {
    $form['global']['global_optout']['#description'] = 'Un-check this box and save to re-activate all current subscriptions.';
  }
  $form['global']['global_optout_submit'] = array('#type' => 'submit', '#value' => t(MAILOUT_SUBMIT_SAVE_GLOBAL));

  if (count($select_other_subs)) {    
    $form['more'] = array('#title' => 'Other subscriptions', '#tree' => FALSE, '#type' => 'fieldset', '#collapsible' => TRUE, '#collapsed' => TRUE);
    $form['more']['global_lists'] = array(
      '#type' => 'select',
      '#multiple' => TRUE,
      '#options' => $select_other_subs,
      '#title' => t('Other possible subscriptions'),
      '#description' => 'Select some lists, and type a \'Display name\' to add this user to more mailing lists.',
    );
    $form['more']['global_lists_submit'] = array('#type' => 'submit', '#value' => t(MAILOUT_SUBMIT_SAVE_NEW));
  }
  $form['html'] = array('#title' => 'E-mail format', '#tree' => FALSE, '#type' => 'fieldset', '#collapsible' => TRUE, '#collapsed' => TRUE);
  $form['html']['global_html'] = array('#type' => 'select', '#title' => '', '#multiple' => FALSE, '#options' => array(MAILOUT_FORMAT_PLAIN => 'Plain', MAILOUT_FORMAT_HTML => 'HTML'), '#default_value' => $subs[$sub_id]->html, '#description' => t('Select a Plain/HTML setting. Use \'Plain text\' for subscribers who do not want HTML format emails.'));
  $form['html']['global_html_submit'] = array('#type' => 'submit', '#value' => t(MAILOUT_SUBMIT_SAVE_HTML));
    
  // OK. I'm simplifying this form for a moment.
  // Too much going on.
  unset($form['html']);
  unset($form['display']);
  
  return $form;
}

function theme_mailout_list_sub_edit_form($form) {
  if ($form['global']['global_optout']['#default_value']) {
    drupal_set_message(t('Warning: This subscriber is currently not receiving emails because they are globally opted-out.'), 'warning');
  }
  
  if (isset($form['apply_to_queues'])) {
    $problems = mailout_stats_not_sent($form['global_email']['#value']);
    foreach ($problems AS $mmid) {
      $text = l($mmid['subject'], 'admin/content/mailout/stats/summary/'. $mmid['mlid'] .'/'. drupal_urlencode($form['global_email']['#value']));
      $text .= ' '. l('(Process queue)', 'admin/content/mailout/stats/cron/'. $mmid['mlid']);
      $links[] = $text;
    }
    drupal_set_message(t(
      'Warning: This subscriber has emails queued up which have not been sent. !queues This might indicate a problem with the subscriber\'s email address. You can check the box labelled \'Fix queue issues\' to push your email changes to the queue.',
      array('!queues' => theme('item_list', $links))
    ), 'warning');
  }

  $output = '';
  foreach (element_children($form['lists']) AS $key) {
    $rows[] = array(
      drupal_render($form['lists'][$key]['list_link']),
      drupal_render($form['lists'][$key]['sub_email']),
      drupal_render($form['lists'][$key]['sub_name']),
      drupal_render($form['lists'][$key]['sub_html']),
      drupal_render($form['lists'][$key]),
    );
  }
  if (!count($rows)) {
    $rows[] = array(array('data' => t('No subscriptions'), 'colspan' => 4));
  }
  
  $output .= theme('table', array('List', 'E-mail', 'Display name', 'Format', 'Delete'), $rows);
  if (isset($form['apply_to_queues'])) { 
    $output .= drupal_render($form['apply_to_queues']);
  }
  $output .= drupal_render($form['submit']);
  $output .= drupal_render($form);
  return $output;        
}

/*
 * Validation mailout_list_sub_edit_form
 */
function mailout_list_sub_edit_form_validate($form_id, $form_values) {  
//  dpr($form_values); die();
  // Validate email addresses.
  switch ($form_values['op']) {
    case t(MAILOUT_SUBMIT_SAVE_GLOBAL):
      break;
    case t(MAILOUT_SUBMIT_SAVE_NAME):
      if (!($form_values['global_name'])) {
        form_set_error("global_name", t('A new display name field is required to update all subscriptions.'));
      }
      break;
    case t(MAILOUT_SUBMIT_SAVE_NEW):
      break;
    case t(MAILOUT_SUBMIT_SAVE_HTML):
      break;
    case t(MAILOUT_SUBMIT_SAVE_LISTS):
      foreach ($form_values['lists'] AS $key => $data) {
        if ($message = user_validate_mail($data['sub_email'])) {
          form_set_error("lists][$key][sub_email", $message);
        }
        if (!($data['sub_name'])) {
          form_set_error("lists][$key][sub_name", t('Display name field is required.'));
        }
      }
  }
}

/*
 * Form submission for mailout_list_sub_edit_form
 */
function mailout_list_sub_edit_form_submit($form_id, $form_values) {  
  switch ($form_values['op']) {
    case t(MAILOUT_SUBMIT_SAVE_GLOBAL):
      // Set opt-out in or out.
      if ($form_values['global_optout']) {
        mailout_optout($form_values['global_email'], $form_values['global_bestname']);
      }
      else {
        mailout_optout_cancel($form_values['global_email']);
      }
      break;
    case t(MAILOUT_SUBMIT_SAVE_NAME):
      if (count($form_values['lists'])) {
        foreach ($form_values['lists'] AS $key => $sub_data) {
          mailout_list_update_subscriber($sub_data['sub_email'], $form_values['global_name'], $sub_data['sub_html'], NULL, $sub_data['sub_id']);
        }
      }
      break;
    case t(MAILOUT_SUBMIT_SAVE_NEW):
      if (count($form_values['global_lists'])) {
        // User is adding subscriber to more lists.
        foreach ($form_values['global_lists'] AS $list_id) {
          $name = ($form_values['global_name']) ? $form_values['global_name'] : $form_values['global_bestname'];
          mailout_list_add_subscriber($form_values['global_email'], $name, $form_values['global_bestformat'], $list_id);
        }
      }
      break;
    case t(MAILOUT_SUBMIT_SAVE_HTML):
      if (count($form_values['lists'])) {
        foreach ($form_values['lists'] AS $key => $sub_data) {
          mailout_list_update_subscriber($sub_data['sub_email'], $sub_data['sub_name'], $form_values['global_html'], NULL, $sub_data['sub_id']);
        }
      }
      drupal_set_message($form_values['global_email'] . t(' email format changed globally.'));
      break;
    case t(MAILOUT_SUBMIT_SAVE_LISTS):
      // User is updating list values
      foreach ($form_values['lists'] AS $key => $sub_data) {
        if ($sub_data['sub_delete']) {
          // Process a delete select box.
          mailout_list_remove_subscriber(NULL, NULL, $sub_data['sub_id']);
          drupal_set_message($sub_data['old_email'] .' removed from \''. $sub_data['list_name'] .'\'');

          // We  might have removed the subscription that we are using for an id, so get redirect to an existing sub id.
          if ($path = mailout_list_subscription_path($sub_data['old_email'])) {
            return $path;
          }
          else {
            return 'admin/content/mailout/lists/overview/'. $sub_data['list_id'];
          }
        }
        elseif ($sub_data['sub_email'] != $sub_data['old_email'] || $sub_data['sub_name'] != $sub_data['old_name'] || $sub_data['sub_html'] != $sub_data['old_html']) {
          $used_new_email = $sub_data['sub_email'];
          // Change values.
          mailout_list_update_subscriber($sub_data['sub_email'], $sub_data['sub_name'], $sub_data['sub_html'], NULL, $sub_data['sub_id']);
          drupal_set_message($sub_data['sub_email'] .' updated for \''. $sub_data['list_name'] .'\'');
        }
        if (isset($used_new_email) && $form_values['apply_to_queues']) {
          $sql = "UPDATE {mailout_message_log} SET email_to = '%s' WHERE SENT = 0 AND email_to LIKE '%s'";
          db_query($sql, $used_new_email, $sub_data['old_email']);
        }
      }
  }
  return 'admin/content/mailout/lists/edit/sub/'. $form_values['edited'];
}

/**
 * Get an subscription id or edit path for administration purposes, by passing an email address.
 */
function mailout_list_subscription_path($email, $id_only = FALSE) {
  $subs = mailout_list_load_subscribers(FALSE, $email);
  if (count($subs)) {
    // One link serves all.
    $subscription = array_pop($subs);
    return ($id_only) ? $subscription->id : 'admin/content/mailout/lists/edit/sub/'. $subscription->id;
  }
  else {
    return FALSE;
  }
}


/**
 * Add or update a mailing list. 
 *
 * @param $values is like $form_values
 * -- optional 'id' to update an existing list, will be a new list if id is not set.
 * -- 'list_name' is the list name
 */
function mailout_list_update($values) {  
  if (!isset($values['list_id']) || $values['list_id'] == 'new') {
    db_query("INSERT INTO {mailout_list} (list_name, public) VALUES ('%s', %d)", $values['list_name'], $values['public']);
    return t("Added new list '%list'", array('%list' => $values['list_name']));
  }
  else {
    db_query("UPDATE {mailout_list} SET list_name = '%s', public = %d WHERE `id` = %d", $values['list_name'], $values['public'], $values['list_id']);
    return t("Updated list '%list'", array('%list' => $values['list_name']));
  }
}

// ---------------------------------------------------------------------------
// Mailout list stats

/**
 * Overview of all mailouts, with links to stat pages
 */
function mailout_stats_summary() {

  $id = arg(5);
  if (!$item = db_fetch_object(db_query('SELECT l.mlid, l.email_subject, count(ml.mmid) as num_mails, sum(case when ml.sent = 1 then 1 else 0 end) as sent, sum(case when ml.opened = 1 then 1 else 0 end) as opened, l.mlid, l.timestamp as timestamp FROM {mailout_log} as l join {mailout_message_log} as ml on ml.mlid = l.mlid WHERE l.mlid = %d GROUP BY l.mlid ORDER BY timestamp desc', $id))) {  
    drupal_goto('admin/content/mailout/stats');
  }

  drupal_set_title($item->email_subject);
  $output = '<p><strong>Total</strong>: '. $item->num_mails .', <strong>Sent</strong>: '. $item->sent .', <strong>Opened</strong>: '. $item->opened .', <strong>Time</strong>: '. $item->timestamp .'</p>';

  // Add a form for searching subscribers in the current list.
  $search_text = (arg(6)) ? arg(6) : NULL;
  // Not sure if this is right way, but $_GET['q'] won't work because it will have extra values sometimes.
  $return = arg(0) .'/'. arg(1) .'/'. arg(2) .'/'. arg(3) .'/'. arg(4) .'/'. arg(5);
  $search_subscribers_form = drupal_get_form('mailout_subscribers_search_form', $return, $search_text);
  
  $all_emails = db_query('SELECT * FROM {mailout_message_log} WHERE mlid = %d ORDER BY email_to', $item->mlid);
  
  $header = array(
    array('data' => t('Subscriber'), 'field' => 'name', 'sort' => 'asc'),
    array('data' => t('Mail'), 'field' => 'email_to'),
    array('data' => t('Sent'), 'field' => 'sent'),
    array('data' => t('Opened'), 'field' => 'opened'),
  );
  $sql = 'SELECT * FROM {mailout_message_log} WHERE mlid = %d ';
  $params[] = $item->mlid;
  if ($search_text) {
    $sql .= ' AND (name LIKE \'%%%s%%\' OR email_to LIKE \'%%%s%%\') ';
    $params = array($item->mlid, $search_text, $search_text);
  }
  $sql .= tablesort_sql($header);

//  $result = db_queryd($sql, $params);
  $result = pager_query($sql, 50, 0, NULL, $params);
  
  while ($email = db_fetch_array($result)) {
    // Get an appropriate user/subscriber link.
    $subscriber = mailout_list_load_subscribers(FALSE, $email['email_to']);
    if (!count($subscriber)) {
      $subscriber = user_load(array('mail' => $email['email_to']));
      $subscriber = l($email['name'], 'user/'. $user->uid);
    }
    else {
      $subscriber = array_pop($subscriber);
      $subscriber = l($email['name'], 'admin/content/mailout/lists/edit/sub/'. $subscriber->id);
    }
    // Do the row data.
    $sent = ($email['sent']) ? theme('image', 'misc/watchdog-ok.png', t('OK'), t('E-mail has been sent')) : '--';
    $opened = ($email['opened']) ? theme('image', 'misc/watchdog-ok.png', t('OK'), t('Confirmed - E-mail has been opened')) : '';
    $rows[] = array($subscriber, $email['email_to'], $sent, $opened);
  }
  $output .= $search_subscribers_form . theme('table', $header, $rows) . theme('pager', NULL, 50, 0, array());
  return $output;  
}

/**
 * Delete all statistics and other information associated with a mailout.
 */
function mailout_stats_delete_form($mlid) {
  $log = db_fetch_object(db_query('SELECT mlid, email_subject FROM {mailout_log} AS ml WHERE mlid = %d', $mlid));
  
  $form['mlid'] = array('#type' => 'value', '#value' => $log->mlid);
  $form['subject'] = array('#type' => 'value', '#value' => $log->email_subject);
  $form['info'] = array('#value' => '<p>'. t('This will remove all log and tracking information, and will abort any emails that have not been sent yet.') .'</p>');

  $form = confirm_form($form, t('Delete Mailout queue and statistics?'), 'admin/content/mailout/stats', t("The statistics for '%mailout_log' will be removed forever.", array('%mailout_log' => $log->email_subject)), t('Delete'), t('Cancel'));  
  return $form;
}

function mailout_stats_delete_form_submit($form_id, $form_values) {
  db_query('DELETE FROM {mailout_log} WHERE mlid = %d', $form_values['mlid']);
  db_query('DELETE FROM {mailout_message_log} WHERE mlid = %d', $form_values['mlid']);
  drupal_set_message(t('All queued mails, log data and statistics data have been deleted for @mlid', array('@mlid' => $form_values['subject'])));
  return 'admin/content/mailout/stats';
}

/**
 * Display statistics about mailouts that have been sent
 */
function mailout_stats(){
  $header_row = array('ID', 'Subject', 'Messages', 'Sent', 'Read', 'Time', 'Operations');
  //get mailout stats
  $result = db_query('SELECT l.mlid, l.email_subject, count(ml.mmid) as num_mails, sum(case when ml.sent = 1 then 1 else 0 end) as sent, sum(case when ml.opened = 1 then 1 else 0 end) as opened, l.mlid, l.timestamp as timestamp FROM {mailout_log} as l join {mailout_message_log} as ml on ml.mlid = l.mlid GROUP BY l.mlid ORDER BY timestamp desc');
  
  $table_rows = array();
  while ($row = db_fetch_array($result)) {
    $row['email_subject'] = l($row['email_subject'], 'admin/content/mailout/stats/summary/'. $row['mlid']);
    if ($row['num_mails'] > 0 && $row['num_mails'] == $row['sent']) {
      $row['sent'] = theme('image', 'misc/watchdog-ok.png', t('OK'), t('All emails have been sent successfully.'));
    }
    $row['operations'] = l(t('Purge'), 'admin/content/mailout/stats/delete/'.  $row['mlid']); 
    $table_rows[] = $row;
  }  
  $output .= theme_table($header_row, $table_rows);
  
  return $output;
}

/**
 * This function tells us whether there are any existing deliveries in any queue that have not been sent.
 */
function mailout_stats_not_sent($email) {
  $sql = "SELECT ml.email_subject AS subject, mml.* FROM {mailout_message_log} AS mml INNER JOIN {mailout_log} AS ml ON mml.mlid = ml.mlid WHERE mml.sent = 0 AND mml.email_to LIKE '%s'";
  $problems = array();
  $result = db_query($sql, $email);
  while ($row = db_fetch_array($result)) {
    $problems[] = $row;
  }
  return $problems;
}


// ---------------------------------------------------------------------------
// Mailout list templates

/**
 * An overview of all mailing lists that are set up.
 */
function mailout_templates() {
  foreach (mailout_get_templates() AS $template) {
    $rows[] = array(
      l($template->subject, 'admin/content/mailout/templates/edit/'. $template->mtid),
      l('Preview', 'admin/content/mailout/templates/preview/'. $template->mtid) .', '. 
      l('Edit', 'admin/content/mailout/templates/edit/'. $template->mtid) .', '.
      l('Clone', 'admin/content/mailout/templates/new/'. $template->mtid) .', '. 
      l('Delete', 'admin/content/mailout/templates/delete/'. $template->mtid),

//      l('Delete', 'admin/content/mailout/templates/delete/'. $template->mtid) .', '. 
//      l('Send', 'admin/content/mailout/send/'. $template->mtid),
    );
  }
  $rows[] = array(array('data' => l('<em>New</em>', 'admin/content/mailout/templates/new', array(), NULL, NULL, FALSE, TRUE), 'colspan' => 2,));
  return theme('table', array('Template', 'Operations'), $rows);
}

/**
 * Edit or create a mailout template.
 */
function mailout_template_edit_form() {
  $mtid = arg(5);
  $action = arg(4);
  if (arg(4) == 'new' && $mtid) {
    // New from existing.
    $template = mailout_template_load($mtid);
  }
  elseif (arg(4) == 'edit' && $mtid) {
    $template = mailout_template_load($mtid);
  }
  else {
    // New from scratch.
    $template = (object)array(
      'mtid' => 0, 'subject' => 'New template', 'email' =>  variable_get('site_name', 'my site.'), 'email' => '"'. variable_get('site_name', 'my site.') .'" <'. variable_get('site_mail', '') .'>', 'stat_track' => TRUE, 'opt_out' => TRUE,
      'header' => '', 'header_format' => variable_get('filter_default_format', 1), 'body' => '', 'body_format' => variable_get('filter_default_format', 1), 'footer' => '', 'footer_format' => variable_get('filter_default_format', 1),
    );
    $action = 'new';
  }

  drupal_set_title($template->subject);
  
  $form['mtid'] = array(
    '#type'=> 'hidden',
    '#value' => $mtid,
  );
  $form['action'] = array(
    '#type'=> 'hidden',
    '#value' => $action,
  );
  $form['subject'] = array(
    '#type'=> 'textfield',
    '#title' => t('E-mail subject'),
    '#default_value' => $template->subject,
    '#required' => TRUE,
  );
  $form['email'] = array(
    '#type'=> 'textfield',
    '#title' => t('From address'),
    '#default_value' => $template->email,
  );  
  $form['stat_track'] = array(
    '#type' => 'select',
    '#title' => t('Track statistics'),
    '#options' => array('0' => 'No', '1' => 'Yes'),
    '#required' => FALSE,
    '#default_value' => $template->stat_track,
    '#description' => t('Should a statistics tracking image be embeded emails? (only works if sending in HTML mode)'),
  ); 
  $form['opt_out'] = array(
    '#type' => 'select',
    '#title' => t('Opt-out link'),
    '#options' => array('0' => 'No', '1' => 'Yes'),
    '#required' => FALSE,
    '#default_value' => $template->opt_out,
    '#description' => t('Should an opt out link be appended to the bottom of emails?'),
  );

  $form['html'] = array(
    '#type' => 'fieldset',
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
    '#title' => 'HTML Template',
  );
  $form['html']['mailout_header'] = array(
    '#type' => 'textarea',
    '#title' => t('Header'),
    '#rows' => 7,
    '#weight' => 1,
    '#default_value' => $template->header,
  );
//  $form['html']['header_format'] = filter_form(variable_get('filter_default_format', 1), 2, $parents = array('html', 'format_header'));
  $form['html']['mailout_body'] = array(
    '#type' => 'textarea',
    '#title' => t('Body'),
    '#description' => t('This can be altered by the person sending the email.'),
    '#rows' => 10,
    '#weight' => 3,
    '#default_value' => $template->body,
  );
//  $form['html']['body_format'] = filter_form(variable_get('filter_default_format', 1), 4, $parents = array('html', 'format_body'));
  $form['html']['mailout_footer'] = array(
    '#type' => 'textarea',
    '#title' => t('Footer'),
    '#rows' => 7,
    '#weight' => 5,
    '#default_value' => $template->footer,
  );   
//  $form['html']['footer_format'] = filter_form(variable_get('filter_default_format', 1), 6, $parents = array('html', 'format_footer'));

  preg_match_all('/\s*(.*)\s*\{\s*(.*)\s*\}/sU', $template->style, $matches, PREG_SET_ORDER);
  $style_msg = '';
  foreach ($matches AS $key => $match) {
    $tag = trim($match[1]);
    $style = str_replace(array("\n", "\r"), ' ', $match[2]);
    $style_msg .= "<$tag style=\"$style\">&lt;$tag&gt;Example&lt;/$tag&gt;</$tag> \n";
  }
  $style_msg = theme('fieldset', array('#title' => 'Style preview', '#collapsible' => FALSE, '#value' => $style_msg,));

  $form['html']['style'] = array(
    '#type' => 'textarea',
    '#title' => t('Style'),
    '#description' => t('Define some styles here for common elements like: H1, H2, P, A, ... These styles will be embedded when you send your email. &lt;img&gt;, class and id selectors do not work currently.') . $style_msg,
    '#rows' => 6,
    '#weight' => 6,
    '#default_value' => $template->style,
  );   

  $form['submit'] = array('#type' => 'submit', '#value' => 'Save');
  
  return $form;
}

/**
 * Edit or create a mailout template.
 */
function mailout_template_edit_form_submit($form_id, $form_values) {
  
  // Make our special named (for wysiwyg element distinction) form elements look like db cols again.
  $form_values['header'] = $form_values['mailout_header'];
  $form_values['body'] = $form_values['mailout_body'];
  $form_values['footer'] = $form_values['mailout_footer'];
  // Save data.
  if ($form_values['action'] == 'new') {
    $mtid = mailout_template_insert($form_values);
  }
  else {
    $mtid = mailout_template_update($form_values['mtid'], $form_values);
  }
  
  drupal_set_message(t('The template "@subject" has been saved', array('@subject' => $form_values['subject'])));
    return 'admin/content/mailout/templates';
}


/**
 * Implementation of hook_insert()
 * for content type mailout_template
 */
function mailout_template_insert($data) {
  db_query("INSERT INTO {mailout_template} (subject, email, opt_out, stat_track, header, body, footer, style) VALUES ('%s', '%s', %d, %d, '%s', '%s', '%s', '%s')",
    $data['subject'], $data['email'], $data['opt_out'], $data['stat_track'], $data['header'], $data['body'], $data['footer'], $data['style']);
  return db_result(db_query("SELECT MAX(mtid) FROM {mailout_template}"));
}

/**
 * Implementation of hook_update()
 * for content type mailout_template
 */
function mailout_template_update($mtid, $data) {
  db_query("UPDATE {mailout_template} SET subject = '%s', email = '%s', opt_out = %d, stat_track = %d, header = '%s', body = '%s', footer = '%s', style = '%s' WHERE mtid = %d",
     $data['subject'], $data['email'], $data['opt_out'], $data['stat_track'], $data['header'], $data['body'], $data['footer'], $data['style'], $mtid);
  return $form_values['mtid'];
}

/**
 * Mailout template edit form, for editing the name, maybe more later.
 */
function mailout_template_delete_form() {
  $mtid = arg(5);  
  $all_templates = mailout_get_templates();  
  $form['mtid'] = array('#type' => 'value', '#value' => $mtid);
  $form['subject'] = array('#type' => 'value', '#value' => $all_templates[$mtid]->subject);

  return confirm_form($form, t('Delete Mailout list?'), 'admin/content/mailout/templates', t("The list '%list' will be removed forever.", array('%template' => $all_templates[$mtid]->subject)), t('Delete'), t('Cancel'));
}

function mailout_template_delete_form_submit($form_id, $form_values) {
  db_query("DELETE FROM {mailout_template} WHERE mtid = %d", $form_values['mtid']);
  drupal_set_message(t("The Mailout template '%template' has been deleted.", array('%template' => $form_values['subject'])));
  return 'admin/content/mailout/templates'; 
}

/**
 * Implementation of hook_load()
 * for content type mailout_template
 */
function mailout_template_load($mtid){
  $template = db_fetch_object(db_query('SELECT * FROM {mailout_template} WHERE mtid = %d', $mtid));
  return $template;
}

// ---------------------------------------------------------------------------
// Mailout files admin


/**
 * Attachment handling is simple at this time and could be fleshed out further.
 */
function mailout_attachments_form() {

  // Make sure our storage is set up.
  $dir = file_directory_path() .'/mailout';
  file_check_directory($dir, FILE_CREATE_DIRECTORY);
  $dir .= '/mtid-all';
  file_check_directory($dir, FILE_CREATE_DIRECTORY);
  
  // This form is a file upload form.
  $form['#attributes']['enctype'] = 'multipart/form-data';

  $mailout_templates = mailout_get_templates();

  $options = array();
  $options[$template->mtid] = array();
  
  // Set up the 'all' files listing
  $found_attachments = file_scan_directory(file_directory_path() .'/mailout/mtid-all', '.*');
  $template_list = array('all' => 'All templates');
  if (count($found_attachments)) {
    $form['mtid-all'] = array('#type' => 'fieldset', '#title' => 'All templates', '#collapsible' => TRUE, '#collapsed' => FALSE,);
    foreach ($found_attachments AS $file) {
      $options['all'][$file->filename] = l($file->basename, $file->filename);
    }
    $form['mtid-all']['all'] = array(
      '#type' => 'checkboxes',
      '#options' => $options['all'],
    );
  }
  
  // Set up the files listing for each templates
  foreach ($mailout_templates AS $template) {
    $template_list[$template->mtid] = $template->subject;
    $options[$template->mtid] = array();
    $dir = file_directory_path() .'/mailout/mtid-'. $template->mtid;
    file_check_directory($dir, FILE_CREATE_DIRECTORY);
    $found_attachments = file_scan_directory($dir, '.*');
    if (count($found_attachments)) {
      $form['mtid-'. $template->mtid] = array('#type' => 'fieldset', '#title' => $template->subject, '#collapsible' => TRUE, '#collapsed' => FALSE,);
      foreach ($found_attachments AS $file) {
        $options[$template->mtid][$file->filename] = l($file->basename, $file->filename);
      }
      $form['mtid-'. $template->mtid][$template->mtid] = array(
        '#type' => 'checkboxes',
        '#options' => $options[$template->mtid],
      );
    }
  }

  $form['delete'] = array(
    '#type' => 'submit',
    '#value' => t(MAILOUT_SUBMIT_DELETE_FILES),
  );
  
  // Form for uploading new files.
  $form['file-new'] = array(
    '#type' => 'file',
    '#title' => t('Upload file'),
    '#size' => 40,
    '#description' => t('Browse and select a file to be used as a mailout attachment.'),
  );
  $form['file-template'] = array(
    '#type' => 'select',
    '#multiple' => FALSE,
    '#title' => t('Attach to'),
    '#options' => $template_list,
    '#default_value' => 'all',
    '#required' => TRUE,
    '#description' => t('Choose the template that this file will be associated with. If you want this file to be available for every mailout, chose \'All templates\'.'),
  );

  $form['upload'] = array(
    '#type' => 'submit',
    '#value' => t(MAILOUT_SUBMIT_UPLOAD_FILE),
  );

  return $form;  
}

function mailout_attachments_form_validate($form_id, $form_values) {
  if($form_values['op'] == t(MAILOUT_SUBMIT_UPLOAD_FILE) && !$file = file_check_upload('file-new') ) {
    form_set_error('file-new', t('Please specify a file to upload.'));
  }
}

function mailout_attachments_form_submit($form_id, $form_values) {
  if ($form_values['op'] == t(MAILOUT_SUBMIT_DELETE_FILES)) {
    foreach ($form_values['all'] AS $filename => $delete) {
      if ($filename == $delete) {
        if (file_delete($delete)) {
          drupal_set_message(t('Deleted') . ': '. $delete);
        }
      }
    }
    foreach(mailout_get_templates() AS $template) {
      if (isset($form_values[$template->mtid]) && count($form_values[$template->mtid])) {
        foreach($form_values[$template->mtid] AS $filename => $delete) {
          if ($filename == $delete) {
            if (file_delete($delete)) {
              drupal_set_message(t('Deleted') . ': '. $delete);
            }
          }
        }
      }
    }
  }
  elseif ($form_values['op'] == t(MAILOUT_SUBMIT_UPLOAD_FILE)) {
    $dir = file_directory_path() .'/mailout/mtid-'. $form_values['file-template'];    
    if ($save = file_save_upload('file-new', $dir, TRUE)) {
      drupal_set_message(t('The file was uploaded.'));
    }
    else {
      drupal_set_message(t('There was an error uploading the file.'));
    }

  }
  
 // drupal_set_message(t('The file )));
  return 'admin/content/mailout/files';
}

/**
 * Menu callback: run cron manually via admin
 * Useful for testing and outputting errors without triggering the rest of cron.
 */
function mailout_stats_cron() {
  if (arg(5)) {
    mailout_send_cron(arg(5));
  }
  else {
    mailout_send_cron();
  }
}
